/* See LICENSE file for copyright and license details. */
#include <X11/XF86keysym.h>
/* appearance */
static int swallowfloating    = 0;        /* 1 means swallow floating windows by default */
static unsigned int borderpx  = 2;        /* border pixel of windows */
static unsigned int snap      = 32;       /* snap pixel */
static unsigned int gappih    = 20;       /* horiz inner gap between windows */
static unsigned int gappiv    = 10;       /* vert inner gap between windows */
static unsigned int gappoh    = 10;       /* horiz outer gap between windows and screen edge */
static unsigned int gappov    = 30;       /* vert outer gap between windows and screen edge */
static const int attachbelow = 1;    /* 1 means attach after the currently active window */
static int smartgaps          = 1;        /* 1 means no outer gap when there is only one window */
static int showbar            = 1;        /* 0 means no bar */
static int topbar             = 0;        /* 0 means bottom bar */
static unsigned int systraypinning = 0;   /* 0: sloppy systray follows selected monitor, >0: pin systray to monitor X */
static unsigned int systrayonleft = 0;    /* 0: systray in the right corner, >0: systray on left of status text */
static unsigned int systrayspacing = 2;   /* systray spacing */
static int systraypinningfailfirst = 1;   /* 1: if pinning fails, display systray on the first monitor, False: display systray on the last monitor*/
static int showsystray        = 0;        /* 0 means no systray */
static int usealtbar          = 1;        /* 1 means use non-dwm status bar */
static char *altbarclass      = "Polybar"; /* Alternate bar class name */
static char *alttrayname      = "tray";    /* Polybar tray instance name */
static char *altbarcmd        = "$HOME/.config/polybar/bar.sh"; /* Alternate bar launch command */

// static char font[]                  = "Iosevka:style=medium:size=12";
// static char dmenufont[]             = "Iosevka:style=medium:size=12";
static char font[]                  = "CozetteVector:size=12";
static char dmenufont[]             = "CozetteVector:size=12";
static const char *fonts[]          = { font, "Symbols Nerd Font:size=12" };
static char normbgcolor[]           = "#222222";
static char normbordercolor[]       = "#444444";
static char normfgcolor[]           = "#bbbbbb";
static char selfgcolor[]            = "#eeeeee";
static char selbordercolor[]        = "#005577";
static char selbgcolor[]            = "#005577";

static char *colors[][3]            = {
       /*               fg           bg           border   */
       [SchemeNorm] = { normfgcolor, normbgcolor, normbordercolor },
       [SchemeSel]  = { selfgcolor,  selbgcolor,  selbordercolor  },
};


/* tagging */
static const char *tags[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };

static const Rule rules[] = {
	/* xprop(1):
	 *	WM_CLASS(STRING) = instance, class
	 *	WM_NAME(STRING) = title
	 */
	/* class     instance  title           tags mask  isfloating  isterminal  noswallow  monitor */
	{ "Gimp",        NULL,     NULL,           0,         1,          0,           0,        -1 },
	{ "Alacritty",   NULL,     NULL,           0,         0,          1,           0,        -1 },
};

/* layout(s) */
static float mfact     = 0.55; /* factor of master area size [0.05..0.95] */
static int nmaster     = 1;    /* number of clients in master area */
static int resizehints = 1;    /* 1 means respect size hints in tiled resizals */
static int lockfullscreen = 1; /* 1 will force focus on the fullscreen window */

#define FORCE_VSPLIT 1  /* nrowgrid layout: force two clients to always split vertically */
#include "vanitygaps.c"

static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "[]=",      tile },    /* first entry is default */
	{ "><>",      NULL },    /* no layout function means floating behavior */
	{ "[M]",      monocle },
	{ "[D]",      deck },
};

/*
 * Xresources preferences to load at startup
 */
ResourcePref resources[] = {
    {"color0", STRING, &normbordercolor},
    {"color8", STRING, &selbordercolor},
    {"color0", STRING, &normbgcolor},
    {"color4", STRING, &normfgcolor},
    {"color0", STRING, &selfgcolor},
    {"color4", STRING, &selbgcolor},
    {"borderpx", INTEGER, &borderpx},
    {"snap", INTEGER, &snap},
    {"showbar", INTEGER, &showbar},
    {"topbar", INTEGER, &topbar},
    {"nmaster", INTEGER, &nmaster},
    {"resizehints", INTEGER, &resizehints},
    {"mfact", FLOAT, &mfact},
    {"gappih", INTEGER, &gappih},
    {"gappiv", INTEGER, &gappiv},
    {"gappoh", INTEGER, &gappoh},
    {"gappov", INTEGER, &gappov},
    {"swallowfloating", INTEGER, &swallowfloating},
    {"smartgaps", INTEGER, &smartgaps},
};

/* key definitions */
#define MODKEY Mod4Mask
#define TAGKEYS(KEY,TAG) \
	{ MODKEY,                       KEY,      view,           {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask,           KEY,      toggleview,     {.ui = 1 << TAG} }, \
	{ MODKEY|ShiftMask,             KEY,      tag,            {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask|ShiftMask, KEY,      toggletag,      {.ui = 1 << TAG} },
#define STACKKEYS(MOD,ACTION) \
	{ MOD,	XK_j,	ACTION##stack,	{.i = INC(+1) } }, \
	{ MOD,	XK_k,	ACTION##stack,	{.i = INC(-1) } }, \
	{ MOD,  XK_v,   ACTION##stack,  {.i = 0 } }, \
	/* { MOD, XK_grave, ACTION##stack, {.i = PREVSEL } }, \ */
	/* { MOD, XK_a,     ACTION##stack, {.i = 1 } }, \ */
	/* { MOD, XK_z,     ACTION##stack, {.i = 2 } }, \ */
	/* { MOD, XK_x,     ACTION##stack, {.i = -1 } }, */

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }
#define TERM(args) { .v = (const char*[]){ "alacritty", "-e", args, NULL }}

/* commands */
static const char *dmenucmd[] = { "dmenu_run", "-fn", dmenufont, "-nb", normbgcolor, "-nf", normfgcolor, "-sb", selbordercolor, "-sf", selfgcolor, NULL };
static const char *termcmd[]  = { "alacritty", NULL };

static const Key keys[] = {
  STACKKEYS(MODKEY,                                        focus)
	STACKKEYS(MODKEY|ShiftMask,                              push)
	/* modifier                     key                      function        argument */
	{ MODKEY,                       XK_d,                    spawn,          {.v = dmenucmd } },
	{ MODKEY,                       XK_Return,               spawn,          {.v = termcmd } },
	{ MODKEY,                       XK_b,                    togglebar,      {0} },
	// { MODKEY,                       XK_j,                 focusstack,     {.i = +1 } },
	// { MODKEY,                       XK_k,                 focusstack,     {.i = -1 } },
	{ MODKEY,                       XK_o,                    incnmaster,     {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_o,                    incnmaster,     {.i = -1 } },
	{ MODKEY,                       XK_h,                    setmfact,       {.f = -0.05} },
	{ MODKEY,                       XK_l,                    setmfact,       {.f = +0.05} },
	{ MODKEY|ShiftMask,             XK_h,                    setcfact,       {.f = +0.25} },
	{ MODKEY|ShiftMask,             XK_l,                    setcfact,       {.f = -0.25} },
	{ MODKEY,                       XK_space,                zoom,           {0} },
	{ MODKEY,                       XK_Tab,                  view,           {0} },
	{ MODKEY,                       XK_q,                    killclient,     {0} },
  { MODKEY,                       XK_f,                    togglefullscr,  {0} },
  /* Layouts */
	{ MODKEY,                       XK_t,                    setlayout,      {.v = &layouts[0]} },
  { MODKEY,                       XK_m,                    setlayout,      {.v = &layouts[2]} },
  { MODKEY|ShiftMask,             XK_m,                    setlayout,      {.v = &layouts[3]} },

	{ MODKEY|ShiftMask,             XK_space,                togglefloating, {0} },
	{ MODKEY,                       XK_0,                    view,           {.ui = ~0 } },
	{ MODKEY|ShiftMask,             XK_0,                    tag,            {.ui = ~0 } },
	{ MODKEY,                       XK_Left,                 focusmon,       {.i = -1 } },
	{ MODKEY,                       XK_Right,                focusmon,       {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_Left,                 tagmon,         {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_Right,                tagmon,         {.i = +1 } },
	TAGKEYS(                        XK_1,                             0)
	TAGKEYS(                        XK_2,                             1)
	TAGKEYS(                        XK_3,                             2)
	TAGKEYS(                        XK_4,                             3)
	TAGKEYS(                        XK_5,                             4)
	TAGKEYS(                        XK_6,                             5)
	TAGKEYS(                        XK_7,                             6)
	TAGKEYS(                        XK_8,                             7)
	TAGKEYS(                        XK_9,                             8)
	{ MODKEY|ShiftMask,             XK_q,                    quit,           {0} },
	{ MODKEY|ShiftMask,             XK_r,                    quit,           {1} },

  /* Applications */
  { MODKEY,                       XK_w,                    spawn,          SHCMD("brave") },
  { MODKEY,                       XK_p,                    spawn,          SHCMD("prismlauncher") },
  { MODKEY|ShiftMask,             XK_p,                    spawn,          TERM("ncspot") },
  { MODKEY,                       XK_e,                    spawn,          SHCMD("thunderbird") },
  { MODKEY|ShiftMask,             XK_e,                    spawn,          SHCMD("evisual") },
  { MODKEY|ShiftMask,             XK_d,                    spawn,          SHCMD("dwm-discord") },
  { MODKEY|ShiftMask,             XK_t,                    spawn,          SHCMD("telegram-desktop") },

  /* Utilities */
  { MODKEY,                       XK_BackSpace,            spawn,          SHCMD("sysact") },
  { 0,                            XK_Print,                spawn,          SHCMD("maim | tee ~/pictures/$(date +%s).png | xclip -selection clipboard -t image/png") },
  { 0|ControlMask,                XK_Print,                spawn,          SHCMD("maim -s | tee ~/pictures/$(date +%s).png | xclip -selection clipboard -t image/png") },
  { MODKEY,                       XK_grave,                spawn,          {.v = (const char *[]){"dmenuunicode", NULL}}},

  /* Media Keys */
  { 0,                            XF86XK_AudioPlay,        spawn,          SHCMD("playerctl play-pause") },
  { 0,                            XF86XK_AudioPrev,        spawn,          SHCMD("playerctl previous") },
  { 0,                            XF86XK_AudioNext,        spawn,          SHCMD("playerctl next") },
  { 0,                            XF86XK_AudioMute,        spawn,          SHCMD("wpctl set-mute @DEFAULT_AUDIO_SINK@ toggle")},
  { 0,                            XF86XK_AudioRaiseVolume, spawn,          SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 3%+")},
  { 0,                            XF86XK_AudioLowerVolume, spawn,          SHCMD("wpctl set-volume @DEFAULT_AUDIO_SINK@ 3%-")},
};

/* button definitions */
/* click can be ClkTagBar, ClkLtSymbol, ClkStatusText, ClkWinTitle, ClkClientWin, or ClkRootWin */
static const Button buttons[] = {
	/* click                event mask      button          function        argument */
	{ ClkLtSymbol,          0,              Button1,        setlayout,      {0} },
	{ ClkLtSymbol,          0,              Button3,        setlayout,      {.v = &layouts[2]} },
	{ ClkWinTitle,          0,              Button2,        zoom,           {0} },
	{ ClkStatusText,        0,              Button2,        spawn,          {.v = termcmd } },
	{ ClkClientWin,         MODKEY,         Button1,        movemouse,      {0} },
	{ ClkClientWin,         MODKEY,         Button2,        togglefloating, {0} },
	{ ClkClientWin,         MODKEY,         Button3,        resizemouse,    {0} },
	{ ClkTagBar,            0,              Button1,        view,           {0} },
	{ ClkTagBar,            0,              Button3,        toggleview,     {0} },
	{ ClkTagBar,            MODKEY,         Button1,        tag,            {0} },
	{ ClkTagBar,            MODKEY,         Button3,        toggletag,      {0} },
};

static const char *ipcsockpath = "/tmp/dwm.sock";
static IPCCommand ipccommands[] = {
  IPCCOMMAND(  view,                1,      {ARG_TYPE_UINT}   ),
  IPCCOMMAND(  toggleview,          1,      {ARG_TYPE_UINT}   ),
  IPCCOMMAND(  tag,                 1,      {ARG_TYPE_UINT}   ),
  IPCCOMMAND(  toggletag,           1,      {ARG_TYPE_UINT}   ),
  IPCCOMMAND(  tagmon,              1,      {ARG_TYPE_UINT}   ),
  IPCCOMMAND(  focusmon,            1,      {ARG_TYPE_SINT}   ),
  IPCCOMMAND(  focusstack,          1,      {ARG_TYPE_SINT}   ),
  IPCCOMMAND(  zoom,                1,      {ARG_TYPE_NONE}   ),
  IPCCOMMAND(  incnmaster,          1,      {ARG_TYPE_SINT}   ),
  IPCCOMMAND(  killclient,          1,      {ARG_TYPE_SINT}   ),
  IPCCOMMAND(  togglefloating,      1,      {ARG_TYPE_NONE}   ),
  IPCCOMMAND(  setmfact,            1,      {ARG_TYPE_FLOAT}  ),
  IPCCOMMAND(  setlayoutsafe,       1,      {ARG_TYPE_PTR}    ),
  IPCCOMMAND(  quit,                1,      {ARG_TYPE_NONE}   )
};

#define SESSION_FILE "/tmp/dwm-session"
